package lichen.utils

import org.slf4j.LoggerFactory
import org.apache.commons.httpclient.methods.{GetMethod, PostMethod}
import org.apache.commons.httpclient.{NameValuePair, HttpStatus, HttpClient}
import org.apache.commons.io.IOUtils
import java.io.InputStream
import io.Codec
import org.apache.commons.httpclient.params.HttpMethodParams

/**
 *
 * http reset client,used as api client
 */
object HttpRestClient{
    //logger
    private val logger = LoggerFactory.getLogger(getClass)
    /** 连接超时 为30秒 **/
    private val CONNECTION_TIMEOUT = 1000 * 30
    /** 读取超时为 1分钟 **/
    private val READ_TIMEOUT = 1000 * 60
    private final val LICHEN_USER_AGENT = "lichen/1.0"

    /**
     * 通过get请求一个url，返回得到的内容
     * @param url 请求的网址
     * @param params 请求时候的参数
     * @param headers 请求时候的头信息
     * @param encoding 编码
     * @return 请求之后的内容
     */
    def get(url: String,
            params: Option[Map[String, String]] = None,
            headers: Option[Map[String, String]] = None,
            encoding: Codec = Codec(Codec.UTF8)): String = {
        var method: GetMethod = null;
        try {
            val client = createHttpClient(encoding)

            method = new GetMethod(url);
            //设置查询参数
            method.setQueryString(
                params getOrElse (Map[String, String]()) map (x => new NameValuePair(x._1, x._2)) toArray
            )
            //设置Http的请求Header
            headers getOrElse (Map[String, String]()) foreach (x => method.setRequestHeader(x._1, x._2))

            val response = client.executeMethod(method);
            if (response == HttpStatus.SC_OK) {
                var stream: InputStream = null
                try {
                    stream = method.getResponseBodyAsStream
                    IOUtils.toString(stream, encoding.name);
                } finally {
                    IOUtils.closeQuietly(stream)
                }

            } else {
                throw new RuntimeException("server返回错误状态：" + method.getStatusLine());
            }
        } finally {
            if (method != null)
                method.releaseConnection();
        }
    }

    /**
     * post方式请求某一个url
     * @param url 请求的网址
     * @param parameters 请求时候的参数
     * @param headers 请求时候的头信息
     * @param encoding 编码
     * @return 服务器返回的内容
     */
    def post(url: String,
             parameters: Option[Map[String, String]] = None, 
             headers: Option[Map[String, String]] = None, 
             encoding: Codec = Codec(Codec.UTF8)): String = {
        var method: PostMethod = null;
        try {
            val client = createHttpClient(encoding)

            method = new PostMethod(url);
            //设置参数
            parameters getOrElse (Map[String, String]()) foreach (x => method.setParameter(x._1, x._2))
            //设置Header
            headers getOrElse (Map[String, String]()) foreach (x => method.setRequestHeader(x._1, x._2))
            val response = client.executeMethod(method);
            if (response == HttpStatus.SC_OK) {
                var stream: InputStream = null
                try {
                    stream = method.getResponseBodyAsStream
                    return IOUtils.toString(stream, encoding.name);
                } finally {
                    IOUtils.closeQuietly(stream)
                }
            } else {
                throw new RuntimeException("server返回错误状态：" + method.getStatusLine());
            }
        } finally {
            if (method != null)
                method.releaseConnection();
        }
    }

    private def createHttpClient(encoding: Codec) = {
        val client = new HttpClient();
        client.getParams.setParameter(HttpMethodParams.HTTP_CONTENT_CHARSET, encoding.name);
        client.getParams.setConnectionManagerTimeout(CONNECTION_TIMEOUT);
        client.getParams.setSoTimeout(READ_TIMEOUT)
        client.getParams.setParameter(HttpMethodParams.USER_AGENT,LICHEN_USER_AGENT)

        client
    }
}